﻿using System;
using System.Collections.Generic;
using StudyCrafter.Assistant.Representation;
using StudyCrafter.Assistant.StudyCrafterScenario;

namespace StudyCrafter.Assistant.Metrics
{
    public class ActionsTakenVariance : SimulatedPlaytraceMetric
    {
        private int thisPlaythroughLength = 0;

        private double avgPlaythroughLength = 0;

        private List<int> playthroughLengths = new List<int>();

        public ActionsTakenVariance(IGraphStore store) : base(store)
        {
        }

        protected override void OnTreeEdge(InteractionEdge edge)
        {
            if (edge.Source.NodeType == InteractionNode.Type.InteractionPoint)
            {
                // This is an action if it was taken from an interaction point.
                thisPlaythroughLength++;
            }
        }

        protected override void OnStartPlaythrough(int playthroughNum)
        {
            thisPlaythroughLength = 0;
        }

        protected override void OnCompletePlaythrough(int playthroughNum)
        {
            playthroughLengths.Add(thisPlaythroughLength);

            avgPlaythroughLength += thisPlaythroughLength / (double)NumPlaythroughs;
        }

        protected override void OnCalculationFinished()
        {
            double sumOfSqrDeviations = 0;
            foreach (int length in playthroughLengths)
            {
                double deviation = Math.Abs(length - avgPlaythroughLength);
                sumOfSqrDeviations += deviation * deviation;
            }

            if (NumPlaythroughs == 1)
            {
                Result = sumOfSqrDeviations;
            }
            else
            {
                Result = sumOfSqrDeviations / (NumPlaythroughs - 1);
            }

            playthroughLengths.Clear();

            base.OnCalculationFinished();
        }
    }
}
