﻿using StudyCrafter.Assistant.Representation;
using StudyCrafter.Assistant.StudyCrafterScenario;

namespace StudyCrafter.Assistant.Metrics
{
    public class ChoiceFrequency : SimulatedPlaytraceMetric
    {

        int numActionTurns = 0;
        int numTotalTurns = 0;

        public ChoiceFrequency(IGraphStore store) : base(store)
        {
        }

        protected override void OnTreeEdge(InteractionEdge edge)
        {
            numTotalTurns++;

            if (edge.Source.NodeType == InteractionNode.Type.InteractionPoint)
            {
                // This is an action if it was taken from an interaction point.
                numActionTurns++;
            }
        }

        protected override void OnStartPlaythrough(int playthroughNum)
        {
            numActionTurns = 0;
            numTotalTurns = 0;
        }

        protected override void OnCompletePlaythrough(int playthroughNum)
        {
            if (numTotalTurns == 0)
            {
                AccumulateMetric(0);
                return;
            }

            AccumulateMetric(numActionTurns / (double)numTotalTurns);
        }

        private void AccumulateMetric(double thisLength)
        {
            Result += thisLength / (double)NumPlaythroughs;
        }
    }
}
