﻿using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;

namespace StudyCrafter.Assistant.Metrics
{
    public class MetricsCSVWriter
    {
        private string ResultFilename
        {
            get
            {
                return "MetricsResults.csv";
            }
        }

        public MetricsCSVWriter()
        {
        }

        public void Write(string path, IDictionary<string, MetricResultsCollection> metricsCollections)
        {
            List<string> headers = metricsCollections.First().Value.Keys.ToList();

            StringBuilder resultStringBuilder = new StringBuilder();

            resultStringBuilder.Append("scenario,");

            foreach (string header in headers)
            {
                resultStringBuilder.AppendFormat("{0},", header);
            }
            resultStringBuilder.AppendLine();

            foreach (KeyValuePair<string, MetricResultsCollection> metricsCollection in metricsCollections)
            {
                resultStringBuilder.AppendFormat("{0},", metricsCollection.Key);
                foreach (string header in headers)
                {
                    double metricValue;
                    if (!metricsCollection.Value.TryGetValue(header, out metricValue))
                    {
                        continue;
                    }

                    resultStringBuilder.AppendFormat("{0},", metricValue);
                }

                resultStringBuilder.AppendLine();
            }

            string finalString = resultStringBuilder.ToString();

            Write(path, finalString);
        }

        private void Write(string path, string finalString)
        {
            var outfile = File.CreateText(Path.Combine(path, ResultFilename));
            outfile.WriteLine(finalString);
            outfile.Close();
        }
    }
}
