﻿using System;
using System.Collections.Generic;
using StudyCrafter.Assistant.StudyCrafterScenario;
using UnityEngine;

namespace StudyCrafter.Assistant.Metrics
{
	public class MetricsCollator
	{
		private IDictionary<string, IScenarioMetric> metrics = new Dictionary<string, IScenarioMetric>();
		private IDictionary<string, CalculationFinishedEvent> resultFuncs = new Dictionary<string, CalculationFinishedEvent>();

		private MetricResultsCollection metricResults = new MetricResultsCollection();

		public MetricResultsCollection Results
		{
			get
			{
				return metricResults;
			}
		}

		public MetricsCollator()
		{
		}

		public void RegisterMetric(string name, IScenarioMetric metric)
		{
			metrics.Add(name, metric);
		}

		private void OnMetricComplete(string name, double result)
		{
			//Debug.Log("Metric: " + name + ": " + result);

			CalculationFinishedEvent resultFunc;
			if (!resultFuncs.TryGetValue(name, out resultFunc))
			{
				return;
			}

			resultFuncs.Remove(name);

			metricResults.Add(name, result);

			IScenarioMetric metric;
			if (!metrics.TryGetValue(name, out metric))
			{
				return;
			}
			metric.CalculationFinished -= resultFunc;

			if (resultFuncs.Count == 0)
			{
				OnFinished();
			}
		}

		public void Run(Scenario scenario)
		{
			if (resultFuncs.Count > 0)
			{
				throw new InvalidOperationException("Already running.");
			}

			RegisterResultFuncs();

			foreach (KeyValuePair<string, IScenarioMetric> metric in metrics)
			{
				try
				{
					metric.Value.Calculate(scenario);

				}
				catch (Exception e)
				{
					throw new Exception("Error while calculating metrics for scenario: " + scenario.ID, e);
				}
			}
		}

		private void RegisterResultFuncs()
		{
			foreach (KeyValuePair<string, IScenarioMetric> metric in metrics)
			{
				CalculationFinishedEvent resultFunc = (result) => OnMetricComplete(metric.Key, result);
				resultFuncs.Add(metric.Key, resultFunc);
				metric.Value.CalculationFinished += resultFunc;
			}
		}

		public event MetricsFinishedEvent Finished;

		public void OnFinished()
		{
			if (Finished != null)
			{
				Finished();
			}
		}
	}
}
