﻿using UnityEngine;
using System.Collections.Generic;
using StudyCrafter.Assistant.StudyCrafterScenario;
using System;

namespace StudyCrafter.Assistant.Metrics
{
	public class MultiScenarioMetricsAnalyzer
	{
		private Func<IDictionary<string, IScenarioMetric>> metricsBuilder;

		private HashSet<string> pendingScenarios = new HashSet<string>();

		private IDictionary<string, MetricResultsCollection> results = new Dictionary<string, MetricResultsCollection>();

		public MetricsFinishedEvent Finished;

		public IDictionary<string, MetricResultsCollection> Results
		{
			get { return results; }
		}

		public MultiScenarioMetricsAnalyzer(Func<IDictionary<string, IScenarioMetric>> metricsBuilder)
		{
			this.metricsBuilder = metricsBuilder;
		}

		public void AnalyzeMetrics(IEnumerable<Scenario> scenarios)
		{
			// Mark all as pending.
			foreach (Scenario scenario in scenarios)
			{
				pendingScenarios.Add(scenario.ID);
			}

			// Analyze them..
			foreach (Scenario scenario in scenarios)
			{
				MetricsCollator metricsCollator = new MetricsCollator();

				IDictionary<string, IScenarioMetric> metrics = metricsBuilder();
				foreach (KeyValuePair<string, IScenarioMetric> metric in metrics)
				{
					metricsCollator.RegisterMetric(metric.Key, metric.Value);
				}

				metricsCollator.Finished += () => OnFinishedScenario(scenario.ID, metricsCollator);

				metricsCollator.Run(scenario);
			}
		}

		private void OnFinishedScenario(string ID, MetricsCollator metricsCollator)
		{
			results.Add(ID, metricsCollator.Results);
			pendingScenarios.Remove(ID);
			if (pendingScenarios.Count == 0)
			{
				OnFinished();
			}
		}

		private void OnFinished()
		{
			if (Finished != null)
			{
				Finished();
			}
		}

		public bool TryGetResults(string scenarioID, out MetricResultsCollection resultMetrics)
		{
			return results.TryGetValue(scenarioID, out resultMetrics);
		}

	}
}
