﻿using System;
using StudyCrafter.Assistant.Representation;
using StudyCrafter.Assistant.StudyCrafterScenario;

namespace StudyCrafter.Assistant.Metrics
{
    public abstract class PerSceneGraphMetric<T> : IScenarioMetric
    {
        protected IGraphStore graphStore;

        public event CalculationFinishedEvent CalculationFinished;

        private int numGraphs;
        private int finishedNum;
        private double metricValue;

        protected PerSceneGraphMetric(IGraphStore store)
        {
            graphStore = store;
        }

        public void Calculate(Scenario scenario)
        {
            // Reset.
            finishedNum = 0;
            metricValue = 0;

            SceneGraph sceneGraph = scenario.GetSceneGraph(graphStore);

            numGraphs = sceneGraph.NumScenes;

            foreach (Scene scene in sceneGraph.GetScenes())
            {
                T graph = GetGraph(scene);

                CalculateForGraph(graph);
            }
        }

        protected abstract T GetGraph(Scene scene);

        protected virtual double ModifyTotal(double total, int numGraphs)
        {
            return total / numGraphs;
        }

        protected abstract void CalculateForGraph(T graph);

        protected virtual void AccumulateMetric(double value)
        {
            finishedNum++;

            metricValue += value;
            if (finishedNum >= numGraphs)
            {
                double finalResult = ModifyTotal(metricValue, numGraphs);
                OnCalculationFinished(finalResult);
            }
        }

        private void OnCalculationFinished(double result)
        {
            if (null != CalculationFinished)
            {
                CalculationFinished(result);
            }
        }
    }
}
