﻿using System.Collections.Generic;
using StudyCrafter.Assistant.Representation;
using StudyCrafter.Assistant.StudyCrafterScenario;

namespace StudyCrafter.Assistant.Metrics
{
    public class RenewalRate : SimulatedPlaytraceMetric
    {
        private HashSet<InteractionEdge> visitedActions = new HashSet<InteractionEdge>();
        private HashSet<InteractionEdge> actionsAcrossSessions = new HashSet<InteractionEdge>();

        public RenewalRate(IGraphStore store) : base(store)
        {
        }

        protected override void OnTreeEdge(InteractionEdge edge)
        {
            if (edge.Source.NodeType == InteractionNode.Type.InteractionPoint)
            {
                visitedActions.Add(edge);
            }
        }

        protected override void OnStartPlaythrough(int playthroughNum)
        {
            visitedActions.Clear();
        }

        protected override void OnCompletePlaythrough(int playthroughNum)
        {
            actionsAcrossSessions.UnionWith(visitedActions);
            AccumulateMetric(visitedActions.Count);
        }

        private void AccumulateMetric(double thisSession)
        {
            Result += thisSession / (double)NumPlaythroughs;
        }

        protected override void OnCalculationFinished()
        {
            // Renewal rate: 1 - (intra-session diversity (average unique actions seen per session) divided by global diversity (global unique actions seen)):
            double globalDiversity = actionsAcrossSessions.Count;
            if (actionsAcrossSessions.Count == 0)
            {
                Result = 0;
            }
            else
            {
                Result /= globalDiversity;
                Result = 1.0 - Result;
                if (Result < 0)
                {
                    Result = 0;
                }
            }

            base.OnCalculationFinished();
        }
    }
}
