﻿using System;

namespace StudyCrafter.Assistant.Representation
{
    public class BaseGraphNode : IGraphNode, IEquatable<BaseGraphNode>
    {
        public string ID
        {
            get
            {
                return properties.Get("ID").Val;
            }
            set
            {
                properties.Set(new StringProperty("ID", value));
            }
        }

        private IPropertyDictionary properties = new PropertyDictionary();

        public IPropertyDictionary Properties
        {
            get
            {
                return properties;
            }
        }

        public BaseGraphNode()
        {
        }

        public BaseGraphNode(string id)
        {
            ID = id;
        }

        public void SetProperty(IProperty property)
        {
            Properties.Set(property);
        }

        public bool TryGetProperty<T>(string propertyName, out T property) where T : IProperty
        {
            return Properties.TryGet<T>(propertyName, out property);
        }

        public bool HasProperty(string propertyName)
        {
            return Properties.Contains(propertyName);
        }

        public T GetProperty<T>(string name) where T : IProperty
        {
            return Properties.Get<T>(name);
        }

        public static bool operator ==(BaseGraphNode node, BaseGraphNode other)
        {
            if (ReferenceEquals(node, null) || ReferenceEquals(other, null))
            {
                return ReferenceEquals(node, null) && ReferenceEquals(other, null);
            }

            return node.Equals(other);
        }

        public static bool operator !=(BaseGraphNode node, BaseGraphNode other)
        {
            return !(node == other);
        }

        public override int GetHashCode()
        {
            return ID.GetHashCode();
        }

        public bool Equals(BaseGraphNode other)
        {
            if (null == other)
            {
                return false;
            }

            return String.Equals(this.ID, other.ID);
        }

        public override bool Equals(object obj)
        {
            if (null == obj)
            {
                return false;
            }

            BaseGraphNode otherBaseGraphNode = obj as BaseGraphNode;
            if (null == obj)
            {
                return false;
            }

            return this.Equals(otherBaseGraphNode);
        }
    }
}
