﻿
using QuickGraph;
using UnityEngine;

namespace StudyCrafter.Assistant.Representation
{
    public static class GraphOwnerExtensions
    {
        public static IMutableBidirectionalGraph<IGraphNode, IEdge<IGraphNode>> ToNodeGraph<T>(this IGraphOwner<T, IEdge<T>> owner) where T : IGraphNode
        {
            return (IMutableBidirectionalGraph<IGraphNode, IEdge<IGraphNode>>)owner.Graph;
        }

        public static TVertex GetOrAddVertex<TVertex, TEdge>(this IGraphOwner<TVertex, TEdge> owner, TVertex vertex) where TEdge : IEdge<TVertex>
        {
            foreach (TVertex graphVertex in owner.Graph.Vertices)
            {
                if (graphVertex.Equals(vertex))
                {
                    return graphVertex;
                }
            }

            if (!owner.Graph.AddVertex(vertex))
            {
                Debug.LogError("Vertex not added!");
                return default(TVertex);
            }

            return vertex;
        }

        public static bool TryGetVertex<TVertex, TEdge>(this IGraphOwner<TVertex, TEdge> owner, TVertex vertex, out TVertex resultVertex) where TEdge : IEdge<TVertex>
        {
            foreach (TVertex graphVertex in owner.Graph.Vertices)
            {
                if (graphVertex.Equals(vertex))
                {
                    resultVertex = graphVertex;
                    return true;
                }
            }

            resultVertex = vertex;
            return false;
        }
    }
}
