﻿using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using StudyCrafter.Assistant.StudyCrafterTypes;

namespace StudyCrafter.Assistant.Representation
{
	public class ChoicesProperty : IProperty
	{
		public class Choice
		{
			public IntProperty Index { get; set; }
			public StringProperty Content { get; set; }
			public BoolProperty Once { get; set; }
			public VariableProperty Owner { get; set; }
			public ExpressionProperty Conditional { get; set; }
		}

		private static Regex choicesRegex;

		static ChoicesProperty()
		{
			// Determine the gap strings use.
			ChoiceStateNode choiceNode = new ChoiceStateNode();
			choicesRegex = new Regex(String.Format(@"(\d+?){0}(.*?){0}(.*?){0}(.*?){0}(.*?){1}", choiceNode.smallGapString, choiceNode.largeGapString));
		}

		public ChoicesProperty(string id, string val)
		{
			ID = id;
			Val = val;
		}

		public string ID
		{
			get;
			private set;
		}

		public string Val
		{
			get
			{
				return ChoicesString;
			}
			private set
			{
				ChoicesString = value;
			}
		}

		private string choicesString;
		public string ChoicesString
		{
			get
			{
				return choicesString;
			}
			private set
			{
				choicesString = value;
				Parse();
			}
		}

		public List<Choice> Choices
		{
			get;
			private set;
		}

		private void Parse()
		{
			List<Choice> choices = new List<Choice>();

			MatchCollection matches = choicesRegex.Matches(ChoicesString);
			foreach (Match match in matches)
			{
				Choice choice = new Choice()
				{
					Index = new IntProperty("index", match.Groups[1].Value),
					Content = new StringProperty("content", match.Groups[2].Value),
					Once = new BoolProperty("once", match.Groups[3].Value),
					Owner = new VariableProperty("owner", match.Groups[4].Value),
					Conditional = new ExpressionProperty("conditional", match.Groups[5].Value)
				};
				choices.Add(choice);
			}

			Choices = choices;
		}
	}
}

