﻿
using System.Text.RegularExpressions;

namespace StudyCrafter.Assistant.Representation
{
    public class VariableProperty : IProperty
    {
        private string variableName;
        private LayoutObjectProperty layoutObjectRef;
        private string varString;

        public static readonly char[] NameDelimiters = new char[] { '{', '}' };

        private static Regex objectPropertyRegex;

        static VariableProperty()
        {
            objectPropertyRegex = new Regex(@"{(\w+)\.(\w+)}");
        }

        public VariableProperty(string id, string val)
        {
            ID = id;
            Val = val;
        }

        public string ID
        {
            get;
            private set;
        }

        public string Val
        {
            get
            {
                return varString;
            }
            private set
            {
                varString = value;
                Parse();
            }
        }

        public string Variable
        {
            get;
            private set;
        }

        public LayoutObjectProperty LayoutObject
        {
            get
            {
                return layoutObjectRef;
            }
            private set
            {
                layoutObjectRef = value;
            }
        }

        public bool IsObjectVar
        {
            get
            {
                return null != LayoutObject;
            }
        }

        private void Parse()
        {
            if (null == varString)
            {
                // We have no value.
                return;
            }

            Match match = objectPropertyRegex.Match(varString);
            if (null != match && match.Success)
            {
                Variable = match.Groups[1].Value;
                LayoutObject = new LayoutObjectProperty("layoutObject", match.Groups[2].Value);
            }
            else
            {
                Variable = varString.Trim(NameDelimiters);
                LayoutObject = null;
            }
        }
    }
}