﻿
using System;
using StudyCrafter.Assistant.Representation;
using StudyCrafter.Assistant.StudyCrafterScenario;
using UnityEngine;

namespace StudyCrafter.Assistant.StudyCrafterLoading
{
    public static class SceneJsonLoadingExtensions
    {
        public static Scene FromSerializationData(this Scene scene, SceneSerializationData sceneData, StudyCrafterLoadHelpers loadHelpers)
        {
            scene.Properties.Set(new BoolProperty("started", sceneData.Started));
            scene.Properties.Set(new StringProperty("manipulator", sceneData.Manipulator));
            try
            {
                JSonScriptGraphBuilder scriptGraphBuilder = new JSonScriptGraphBuilder(sceneData.Scripts, loadHelpers);
                ScriptGraph scriptGraph = scriptGraphBuilder.Build();
                scene.ScriptGraph = scriptGraph.ID;

                JSonLayoutGraphBuilder layoutGraphBuilder = new JSonLayoutGraphBuilder(sceneData.Layout, loadHelpers);
                LayoutGraph layoutGraph = layoutGraphBuilder.Build();
                scene.LayoutGraph = layoutGraph.ID;

                InteractionMapMetaGraphBuilder interactionMapBuilder = new InteractionMapMetaGraphBuilder(scriptGraph, loadHelpers);
                InteractionMap interactionMap = interactionMapBuilder.Build();
                scene.InteractionMap = interactionMap.ID;
            }
            catch (Exception e)
            {
                Debug.LogError("Error while loading scene " + scene.ID);
                throw e;
            }

            return scene;
        }
    }
}
