﻿using System.Text;
using QuickGraph;
using QuickGraph.Algorithms.Services;
using QuickGraph.Graphviz;
using QuickGraph.Graphviz.Dot;
using StudyCrafter.Assistant.Representation;
using StudyCrafter.Assistant.StudyCrafterScenario;

namespace StudyCrafter.Assistant.StudyCrafterSaving
{
    public class ScriptGraphToDotExporter : GraphToDotExporter<ScriptGraph, ScriptNode, IEdge<ScriptNode>>
    {
        public ScriptGraphToDotExporter(string filePath, string fileNamePrefix, ScriptGraph visitedGraph) : base(filePath, fileNamePrefix, visitedGraph)
        {
        }

        public ScriptGraphToDotExporter(string filePath, string fileNamePrefix, IAlgorithmComponent host, ScriptGraph visitedGraph) : base(filePath, fileNamePrefix, host, visitedGraph)
        {
        }

        protected override void FormatVertex(object sender, QuickGraph.Graphviz.FormatVertexEventArgs<ScriptNode> args)
        {
            base.FormatVertex(sender, args);

            args.VertexFormatter.Shape = GraphvizVertexShape.Rectangle;

            args.VertexFormatter.Label = string.Format("<{0} <BR/> <FONT POINT-SIZE=\"9\"> {1} </FONT> >",
                                                       args.VertexFormatter.Label,
                                                       FormatProperties(args.Vertex.Properties));
        }

        private string FormatProperties(IPropertyDictionary properties)
        {
            StringBuilder sb = new StringBuilder();
            foreach (IProperty property in properties.All())
            {
                if (string.IsNullOrEmpty(property.Val))
                {
                    continue;
                }

                switch (property.ID)
                {
                    case "ID":
                    case "type":
                    case "tru":
                    case "index":
                    case "posX":
                    case "posY":
                    case "next":
                        // We don't really care about these properties.
                        break;
                    default:
                        string escapedVal = property.Val.Replace("\"", "&quot;")
                                                        .Replace("&", "&amp;")
                                                        .Replace("<", "&lt;")
                                                        .Replace(">", "&gt;");

                        sb.Append(string.Format("{0}: {1} <BR/>", property.ID, escapedVal));
                        break;
                }
            }

            return sb.ToString();
        }
    }
}
