﻿using System.Collections.Generic;
using System.Linq;
using StudyCrafter.Assistant.Representation;

namespace StudyCrafter.Assistant.StudyCrafterScenario
{
	public class InteractionNode : BaseGraphNode
	{
		public enum Type
		{
			Unknown,
			NonInteraction,
			Event,
			InteractionPoint,
			Feedback,
			Start,
			End
		}

		public InteractionNode.Type NodeType { get; private set; }

		IEnumerable<string> interactionUnits = new List<string>();
		public IEnumerable<string> InteractionUnits
		{
			get
			{
				return interactionUnits;
			}
		}

		public InteractionNode(InteractionNode other)
		{
			NodeType = other.NodeType;
			AssociateWithUnits(other.InteractionUnits);

			Properties.AddAll(other.Properties);
		}

		public InteractionNode(InteractionNode.Type type)
		{
			NodeType = type;
		}

		public InteractionNode(string id, InteractionNode.Type type) : base(id)
		{
			NodeType = type;
		}

		public void AssociateWithUnit(string id)
		{
			AssociateWithUnits(new List<string>() { id });
		}

		public void AssociateWithUnits(IEnumerable<string> ids)
		{
			interactionUnits = interactionUnits.Union(ids);
		}

		public void ClearInteractionUnits()
		{
			interactionUnits = new List<string>();
		}

		public override string ToString()
		{
			return string.Format("{0}: {1}", ID, NodeType);
		}

		public void RecomputeType()
		{
			if (NodeType != Type.Unknown)
			{
				return;
			}

			if (InteractionUnits.Any())
			{
				NodeType = Type.Feedback;
			}
			else
			{
				NodeType = Type.Event;
			}
		}

	}
}
