﻿
using QuickGraph;
using StudyCrafter.Assistant.Representation;
using UnityEngine;

namespace StudyCrafter.Assistant.StudyCrafterScenario
{
    public class LayoutGraph : BaseGraphOwner<LayoutNode, LayoutGraph.Edge>
    {
        public class Edge : QuickGraph.Edge<LayoutNode>
        {
            public Edge(LayoutNode source, LayoutNode target) : base(source, target)
            {
            }

            public Vector3 Offset
            {
                get
                {
                    return Target.Position - Source.Position;
                }
            }
        }

        private IPropertyDictionary properties = new PropertyDictionary();

        public Vector3 Position { get; set; }

        public int UID
        {
            get
            {
                return properties.Get<IntProperty>("uid").IntValue;
            }
            set
            {
                properties.Set(new IntProperty("uid", value));
            }
        }

        public string Name
        {
            get
            {
                return properties.Get<StringProperty>("name").Val;
            }
            set
            {
                properties.Set(new StringProperty("name", value));
            }
        }

        public void UpdateAllEdges()
        {
            // Create edges between all objects that don't have them.
            foreach (LayoutNode node in Graph.Vertices)
            {
                UpdateEdges(node);
            }
        }

        private void UpdateEdges(LayoutNode node)
        {
            foreach (LayoutNode otherNode in Graph.Vertices)
            {
                if (otherNode == node)
                {
                    continue;
                }

                Edge betweenNodesEdge;
                if (!graph.TryGetEdge(node, otherNode, out betweenNodesEdge))
                {
                    betweenNodesEdge = new Edge(node, otherNode);
                    graph.AddEdge(betweenNodesEdge);
                }

                if (!graph.TryGetEdge(otherNode, node, out betweenNodesEdge))
                {
                    betweenNodesEdge = new Edge(otherNode, node);
                    graph.AddEdge(betweenNodesEdge);
                }
            }
        }

        public string Title
        {
            get
            {
                return properties.Get<StringProperty>("title").Val;
            }
            set
            {
                properties.Set(new StringProperty("title", value));
            }
        }

        public string Background
        {
            get
            {
                return properties.Get<StringProperty>("background").Val;
            }
            set
            {
                properties.Set(new StringProperty("background", value));
            }
        }

        public IPropertyDictionary Properties
        {
            get { return properties; }
        }

        public LayoutGraph() : base()
        {
            graph = new BidirectionalGraph<LayoutNode, Edge>();
        }

        public T GetProperty<T>(string name) where T : IProperty
        {
            return properties.Get<T>(name);
        }

        public bool TryGetProperty<T>(string name, out T property) where T : IProperty
        {
            return properties.TryGet<T>(name, out property);
        }

        public void AddNode(LayoutNode node, bool updateEdges = true)
        {
            Graph.AddVertex(node);

            if (updateEdges)
            {
                UpdateEdges(node);
            }
        }

        public int NumNodes
        {
            get
            {
                return graph.VertexCount;
            }
        }
    }
}
