﻿
using System.Collections.Generic;
using QuickGraph;
using StudyCrafter.Assistant.Representation;

namespace StudyCrafter.Assistant.StudyCrafterScenario
{
    public class ScenarioStore
    {
        private IDictionary<string, Scenario> scenarios = new Dictionary<string, Scenario>();

        public IGraphStore GraphStore
        {
            get;
            private set;
        }

        public ScenarioStore(IGraphStore graphStore)
        {
            GraphStore = graphStore;
        }

        public void Add(Scenario scenario)
        {
            scenarios.Add(scenario.ID, scenario);
        }

        public void Remove(Scenario scenario)
        {
            scenarios.Remove(scenario.ID);
        }

        public void Remove(string scenarioID)
        {
            scenarios.Remove(scenarioID);
        }

        public bool TryGet(string scenarioID, out Scenario scenario)
        {
            return scenarios.TryGetValue(scenarioID, out scenario);
        }

        public bool TryGetSceneGraph(string scenarioID, out SceneGraph sceneGraph)
        {
            Scenario scenario;
            if (!TryGet(scenarioID, out scenario))
            {
                sceneGraph = null;
                return false;
            }

            IGraphOwner<Scene, IEdge<Scene>> result;
            if (!GraphStore.TryGetOwner(scenario.SceneGraph, out result))
            {
                sceneGraph = null;
                return false;
            }

            sceneGraph = result as SceneGraph;
            if (sceneGraph == null)
            {
                return false;
            }

            return true;
        }


        public IEnumerable<Scenario> AllScenarios
        {
            get
            {
                return scenarios.Values;
            }
        }

    }
}
