﻿using System;
using UnityEngine;
using UnityEngine.UI;

namespace StudyCrafter.Assistant.UI
{
	public class AnalyzeDirectoryController : MonoBehaviour
	{
		[SerializeField]
		private ScenarioDirectoryAnalyzerComponent analyzer = null;

		[SerializeField]
		private Button button;

		[SerializeField]
		private StatusTextController statusText;

		[SerializeField]
		private PathController metricsExportPathController;

		[SerializeField]
		private PathController projectsDirectoryPathController;

		public AnalyzeDirectoryController()
		{
		}

		public void Start()
		{
			if (null != analyzer)
			{
				analyzer.AnalysisFinishedHandler += OnAnalysisFinished;
				analyzer.ErrorEventHandler += OnAnalysisError;
			}

			if (null != metricsExportPathController)
			{
				if (null != analyzer)
				{
					metricsExportPathController.FilePath = analyzer.MetricsExportPath;
				}
				metricsExportPathController.PathUpdatedListener += OnMetricsExportPathUpdated;
			}

			if (null != projectsDirectoryPathController)
			{
				if (null != analyzer)
				{
					projectsDirectoryPathController.FilePath = analyzer.ProjectsDirectoryPath;
				}
				projectsDirectoryPathController.PathUpdatedListener += OnProjectsDirectoryPathUpdated;
			}
		}

		public void OnClick()
		{
			if (null == analyzer)
			{
				OnAnalysisError("Project analysis controller has no associated analysis component!");
				return;
			}

			button.enabled = false;

			statusText.DisplayMessage("Analyzing projects...");
			analyzer.Analyze();
		}

		void OnAnalysisFinished()
		{
			statusText.DisplayMessage("Finished!");

			button.enabled = true;
		}

		void OnAnalysisError(string error)
		{
			statusText.DisplayError(error);

			button.enabled = true;
		}

		void OnMetricsExportPathUpdated(string exportPath)
		{
			analyzer.MetricsExportPath = exportPath;
		}

		void OnProjectsDirectoryPathUpdated(string projectsPath)
		{
			analyzer.ProjectsDirectoryPath = projectsPath;
		}
	}
}

